# encoding: utf-8
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is
# regenerated.

module Azure::Reservations::Mgmt::V2019_07_19_preview
  #
  # Microsoft Azure Quota Resource Provider.
  #
  class Quota
    include MsRestAzure

    #
    # Creates and initializes a new instance of the Quota class.
    # @param client service class for accessing basic functionality.
    #
    def initialize(client)
      @client = client
    end

    # @return [ReservationsManagementClient] reference to the ReservationsManagementClient
    attr_reader :client

    #
    # Gets the current quota limit and usages for the resource provider for the
    # specified location for the specific resource in the parameter.
    #
    # This API gets the current quota limit and usages for the specific resource
    # for resource provider for the specified location. This response can be used
    # to submit quotaRequests.
    #
    # @param subscription_id [String] Azure subscription id.
    # @param provider_id [String] Azure resource Provider id.
    # @param location [String] Azure region.
    # @param resource_name [String] The Resource name for the specific resource
    # provider, such as SKU name for Microsoft.Compute, pool for Microsoft.Batch.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [CurrentQuotaLimitBase] operation results.
    #
    def list_status(subscription_id, provider_id, location, resource_name, custom_headers:nil)
      response = list_status_async(subscription_id, provider_id, location, resource_name, custom_headers:custom_headers).value!
      response.body unless response.nil?
    end

    #
    # Gets the current quota limit and usages for the resource provider for the
    # specified location for the specific resource in the parameter.
    #
    # This API gets the current quota limit and usages for the specific resource
    # for resource provider for the specified location. This response can be used
    # to submit quotaRequests.
    #
    # @param subscription_id [String] Azure subscription id.
    # @param provider_id [String] Azure resource Provider id.
    # @param location [String] Azure region.
    # @param resource_name [String] The Resource name for the specific resource
    # provider, such as SKU name for Microsoft.Compute, pool for Microsoft.Batch.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [MsRestAzure::AzureOperationResponse] HTTP response information.
    #
    def list_status_with_http_info(subscription_id, provider_id, location, resource_name, custom_headers:nil)
      list_status_async(subscription_id, provider_id, location, resource_name, custom_headers:custom_headers).value!
    end

    #
    # Gets the current quota limit and usages for the resource provider for the
    # specified location for the specific resource in the parameter.
    #
    # This API gets the current quota limit and usages for the specific resource
    # for resource provider for the specified location. This response can be used
    # to submit quotaRequests.
    #
    # @param subscription_id [String] Azure subscription id.
    # @param provider_id [String] Azure resource Provider id.
    # @param location [String] Azure region.
    # @param resource_name [String] The Resource name for the specific resource
    # provider, such as SKU name for Microsoft.Compute, pool for Microsoft.Batch.
    # @param [Hash{String => String}] A hash of custom headers that will be added
    # to the HTTP request.
    #
    # @return [Concurrent::Promise] Promise object which holds the HTTP response.
    #
    def list_status_async(subscription_id, provider_id, location, resource_name, custom_headers:nil)
      fail ArgumentError, 'subscription_id is nil' if subscription_id.nil?
      fail ArgumentError, 'provider_id is nil' if provider_id.nil?
      fail ArgumentError, 'location is nil' if location.nil?
      fail ArgumentError, '@client.api_version is nil' if @client.api_version.nil?
      fail ArgumentError, 'resource_name is nil' if resource_name.nil?


      request_headers = {}
      request_headers['Content-Type'] = 'application/json; charset=utf-8'

      # Set Headers
      request_headers['x-ms-client-request-id'] = SecureRandom.uuid
      request_headers['accept-language'] = @client.accept_language unless @client.accept_language.nil?
      path_template = 'subscriptions/{subscriptionId}/providers/Microsoft.Capacity/resourceProviders/{providerId}/locations/{location}/serviceLimits/{resourceName}'

      request_url = @base_url || @client.base_url

      options = {
          middlewares: [[MsRest::RetryPolicyMiddleware, times: 3, retry: 0.02], [:cookie_jar]],
          path_params: {'subscriptionId' => subscription_id,'providerId' => provider_id,'location' => location,'resourceName' => resource_name},
          query_params: {'api-version' => @client.api_version},
          headers: request_headers.merge(custom_headers || {}),
          base_url: request_url
      }
      promise = @client.make_request_async(:get, path_template, options)

      promise = promise.then do |result|
        http_response = result.response
        status_code = http_response.status
        response_content = http_response.body
        unless status_code == 200
          error_model = JSON.load(response_content)
          fail MsRest::HttpOperationError.new(result.request, http_response, error_model)
        end

        result.request_id = http_response['x-ms-request-id'] unless http_response['x-ms-request-id'].nil?
        result.correlation_request_id = http_response['x-ms-correlation-request-id'] unless http_response['x-ms-correlation-request-id'].nil?
        result.client_request_id = http_response['x-ms-client-request-id'] unless http_response['x-ms-client-request-id'].nil?
        # Deserialize Response
        if status_code == 200
          begin
            parsed_response = response_content.to_s.empty? ? nil : JSON.load(response_content)
            result_mapper = Azure::Reservations::Mgmt::V2019_07_19_preview::Models::CurrentQuotaLimitBase.mapper()
            result.body = @client.deserialize(result_mapper, parsed_response)
          rescue Exception => e
            fail MsRest::DeserializationError.new('Error occurred in deserializing the response', e.message, e.backtrace, result)
          end
        end

        result
      end

      promise.execute
    end

  end
end
