# encoding: utf-8
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is
# regenerated.

module Azure::Reservations::Mgmt::V2019_07_19_preview
  #
  # Microsoft Azure Quota Resource Provider.
  #
  class QuotaRequest
    include MsRestAzure

    #
    # Creates and initializes a new instance of the QuotaRequest class.
    # @param client service class for accessing basic functionality.
    #
    def initialize(client)
      @client = client
    end

    # @return [ReservationsManagementClient] reference to the ReservationsManagementClient
    attr_reader :client

    #
    # Submits a Quota Request for a resource provider at the specified location for
    # the specific resource in the parameter.
    #
    # Submits Quota change request for a resource provider for the specified
    # location for the specific resource in the parameter. To use, first make a Get
    # request to get quota information. This information consists of a list of
    # resources and information regarding those resources. For all the resources in
    # that list which require an update to their quotas, update their limit fields
    # in the response from the Get request to their new values. Then, submit this
    # updated JSON object to this quota request API. This will update the quota to
    # the values specified. The location header in the response will be used to
    # track the status of the quota request. Please check the provisioningState
    # field in the response.
    #
    # @param subscription_id [String] Azure subscription id.
    # @param provider_id [String] Azure resource Provider id.
    # @param location [String] Azure region.
    # @param resource_name [String] The Resource name for the specific resource
    # provider, such as SKU name for Microsoft.Compute, pool for Microsoft.Batch.
    # @param create_quota_request [CurrentQuotaLimitBase] Quota requests payload.
    # @param if_match [String] ETag of the Entity. ETag should match the current
    # entity state from the header response of the GET request or it should be *
    # for unconditional update.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [Object] operation results.
    #
    def create(subscription_id, provider_id, location, resource_name, create_quota_request, if_match, custom_headers:nil)
      response = create_async(subscription_id, provider_id, location, resource_name, create_quota_request, if_match, custom_headers:custom_headers).value!
      response.body unless response.nil?
    end

    #
    # @param subscription_id [String] Azure subscription id.
    # @param provider_id [String] Azure resource Provider id.
    # @param location [String] Azure region.
    # @param resource_name [String] The Resource name for the specific resource
    # provider, such as SKU name for Microsoft.Compute, pool for Microsoft.Batch.
    # @param create_quota_request [CurrentQuotaLimitBase] Quota requests payload.
    # @param if_match [String] ETag of the Entity. ETag should match the current
    # entity state from the header response of the GET request or it should be *
    # for unconditional update.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [Concurrent::Promise] promise which provides async access to http
    # response.
    #
    def create_async(subscription_id, provider_id, location, resource_name, create_quota_request, if_match, custom_headers:nil)
      # Send request
      promise = begin_create_async(subscription_id, provider_id, location, resource_name, create_quota_request, if_match, custom_headers:custom_headers)

      promise = promise.then do |response|
        # Defining deserialization method.
        deserialize_method = lambda do |parsed_response|
          result_mapper = {
            client_side_validation: true,
            required: false,
            serialized_name: 'parsed_response',
            type: {
              name: 'Object'
            }
          }
          parsed_response = @client.deserialize(result_mapper, parsed_response)
        end

        # Waiting for response.
        @client.get_long_running_operation_result(response, deserialize_method)
      end

      promise
    end

    #
    # Submits a Quota Request for a resource provider at the specified location for
    # the specific resource in the parameter.
    #
    # Submits Quota change request for a resource provider for the specified
    # location for the specific resource in the parameter. To use, first make a Get
    # request to get quota information. This information consists of a list of
    # resources and information regarding those resources. For all the resources in
    # that list which require an update to their quotas, update their limit fields
    # in the response from the Get request to their new values. Then, submit this
    # updated JSON object to this quota request API. This will update the quota to
    # the values specified. The location header in the response will be used to
    # track the status of the quota request. Please check the provisioningState
    # field in the response.
    #
    # @param subscription_id [String] Azure subscription id.
    # @param provider_id [String] Azure resource Provider id.
    # @param location [String] Azure region.
    # @param resource_name [String] The Resource name for the specific resource
    # provider, such as SKU name for Microsoft.Compute, pool for Microsoft.Batch.
    # @param create_quota_request [CurrentQuotaLimitBase] Quota requests payload.
    # @param if_match [String] ETag of the Entity. ETag should match the current
    # entity state from the header response of the GET request or it should be *
    # for unconditional update.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [Object] operation results.
    #
    def update(subscription_id, provider_id, location, resource_name, create_quota_request, if_match, custom_headers:nil)
      response = update_async(subscription_id, provider_id, location, resource_name, create_quota_request, if_match, custom_headers:custom_headers).value!
      response.body unless response.nil?
    end

    #
    # @param subscription_id [String] Azure subscription id.
    # @param provider_id [String] Azure resource Provider id.
    # @param location [String] Azure region.
    # @param resource_name [String] The Resource name for the specific resource
    # provider, such as SKU name for Microsoft.Compute, pool for Microsoft.Batch.
    # @param create_quota_request [CurrentQuotaLimitBase] Quota requests payload.
    # @param if_match [String] ETag of the Entity. ETag should match the current
    # entity state from the header response of the GET request or it should be *
    # for unconditional update.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [Concurrent::Promise] promise which provides async access to http
    # response.
    #
    def update_async(subscription_id, provider_id, location, resource_name, create_quota_request, if_match, custom_headers:nil)
      # Send request
      promise = begin_update_async(subscription_id, provider_id, location, resource_name, create_quota_request, if_match, custom_headers:custom_headers)

      promise = promise.then do |response|
        # Defining deserialization method.
        deserialize_method = lambda do |parsed_response|
          result_mapper = {
            client_side_validation: true,
            required: false,
            serialized_name: 'parsed_response',
            type: {
              name: 'Object'
            }
          }
          parsed_response = @client.deserialize(result_mapper, parsed_response)
        end

        # Waiting for response.
        @client.get_long_running_operation_result(response, deserialize_method)
      end

      promise
    end

    #
    # Submits a Quota Request for a resource provider at the specified location for
    # the specific resource in the parameter.
    #
    # Submits Quota change request for a resource provider for the specified
    # location for the specific resource in the parameter. To use, first make a Get
    # request to get quota information. This information consists of a list of
    # resources and information regarding those resources. For all the resources in
    # that list which require an update to their quotas, update their limit fields
    # in the response from the Get request to their new values. Then, submit this
    # updated JSON object to this quota request API. This will update the quota to
    # the values specified. The location header in the response will be used to
    # track the status of the quota request. Please check the provisioningState
    # field in the response.
    #
    # @param subscription_id [String] Azure subscription id.
    # @param provider_id [String] Azure resource Provider id.
    # @param location [String] Azure region.
    # @param resource_name [String] The Resource name for the specific resource
    # provider, such as SKU name for Microsoft.Compute, pool for Microsoft.Batch.
    # @param create_quota_request [CurrentQuotaLimitBase] Quota requests payload.
    # @param if_match [String] ETag of the Entity. ETag should match the current
    # entity state from the header response of the GET request or it should be *
    # for unconditional update.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [Object] operation results.
    #
    def begin_create(subscription_id, provider_id, location, resource_name, create_quota_request, if_match, custom_headers:nil)
      response = begin_create_async(subscription_id, provider_id, location, resource_name, create_quota_request, if_match, custom_headers:custom_headers).value!
      response.body unless response.nil?
    end

    #
    # Submits a Quota Request for a resource provider at the specified location for
    # the specific resource in the parameter.
    #
    # Submits Quota change request for a resource provider for the specified
    # location for the specific resource in the parameter. To use, first make a Get
    # request to get quota information. This information consists of a list of
    # resources and information regarding those resources. For all the resources in
    # that list which require an update to their quotas, update their limit fields
    # in the response from the Get request to their new values. Then, submit this
    # updated JSON object to this quota request API. This will update the quota to
    # the values specified. The location header in the response will be used to
    # track the status of the quota request. Please check the provisioningState
    # field in the response.
    #
    # @param subscription_id [String] Azure subscription id.
    # @param provider_id [String] Azure resource Provider id.
    # @param location [String] Azure region.
    # @param resource_name [String] The Resource name for the specific resource
    # provider, such as SKU name for Microsoft.Compute, pool for Microsoft.Batch.
    # @param create_quota_request [CurrentQuotaLimitBase] Quota requests payload.
    # @param if_match [String] ETag of the Entity. ETag should match the current
    # entity state from the header response of the GET request or it should be *
    # for unconditional update.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [MsRestAzure::AzureOperationResponse] HTTP response information.
    #
    def begin_create_with_http_info(subscription_id, provider_id, location, resource_name, create_quota_request, if_match, custom_headers:nil)
      begin_create_async(subscription_id, provider_id, location, resource_name, create_quota_request, if_match, custom_headers:custom_headers).value!
    end

    #
    # Submits a Quota Request for a resource provider at the specified location for
    # the specific resource in the parameter.
    #
    # Submits Quota change request for a resource provider for the specified
    # location for the specific resource in the parameter. To use, first make a Get
    # request to get quota information. This information consists of a list of
    # resources and information regarding those resources. For all the resources in
    # that list which require an update to their quotas, update their limit fields
    # in the response from the Get request to their new values. Then, submit this
    # updated JSON object to this quota request API. This will update the quota to
    # the values specified. The location header in the response will be used to
    # track the status of the quota request. Please check the provisioningState
    # field in the response.
    #
    # @param subscription_id [String] Azure subscription id.
    # @param provider_id [String] Azure resource Provider id.
    # @param location [String] Azure region.
    # @param resource_name [String] The Resource name for the specific resource
    # provider, such as SKU name for Microsoft.Compute, pool for Microsoft.Batch.
    # @param create_quota_request [CurrentQuotaLimitBase] Quota requests payload.
    # @param if_match [String] ETag of the Entity. ETag should match the current
    # entity state from the header response of the GET request or it should be *
    # for unconditional update.
    # @param [Hash{String => String}] A hash of custom headers that will be added
    # to the HTTP request.
    #
    # @return [Concurrent::Promise] Promise object which holds the HTTP response.
    #
    def begin_create_async(subscription_id, provider_id, location, resource_name, create_quota_request, if_match, custom_headers:nil)
      fail ArgumentError, 'subscription_id is nil' if subscription_id.nil?
      fail ArgumentError, 'provider_id is nil' if provider_id.nil?
      fail ArgumentError, 'location is nil' if location.nil?
      fail ArgumentError, 'resource_name is nil' if resource_name.nil?
      fail ArgumentError, '@client.api_version is nil' if @client.api_version.nil?
      fail ArgumentError, 'create_quota_request is nil' if create_quota_request.nil?
      fail ArgumentError, 'if_match is nil' if if_match.nil?


      request_headers = {}
      request_headers['Content-Type'] = 'application/json; charset=utf-8'

      # Set Headers
      request_headers['x-ms-client-request-id'] = SecureRandom.uuid
      request_headers['If-Match'] = if_match unless if_match.nil?
      request_headers['accept-language'] = @client.accept_language unless @client.accept_language.nil?

      # Serialize Request
      request_mapper = Azure::Reservations::Mgmt::V2019_07_19_preview::Models::CurrentQuotaLimitBase.mapper()
      request_content = @client.serialize(request_mapper,  create_quota_request)
      request_content = request_content != nil ? JSON.generate(request_content, quirks_mode: true) : nil

      path_template = 'subscriptions/{subscriptionId}/providers/Microsoft.Capacity/resourceProviders/{providerId}/locations/{location}/serviceLimits/{resourceName}'

      request_url = @base_url || @client.base_url

      options = {
          middlewares: [[MsRest::RetryPolicyMiddleware, times: 3, retry: 0.02], [:cookie_jar]],
          path_params: {'subscriptionId' => subscription_id,'providerId' => provider_id,'location' => location,'resourceName' => resource_name},
          query_params: {'api-version' => @client.api_version},
          body: request_content,
          headers: request_headers.merge(custom_headers || {}),
          base_url: request_url
      }
      promise = @client.make_request_async(:put, path_template, options)

      promise = promise.then do |result|
        http_response = result.response
        status_code = http_response.status
        response_content = http_response.body
        unless status_code == 200 || status_code == 201
          error_model = JSON.load(response_content)
          fail MsRest::HttpOperationError.new(result.request, http_response, error_model)
        end

        result.request_id = http_response['x-ms-request-id'] unless http_response['x-ms-request-id'].nil?
        result.correlation_request_id = http_response['x-ms-correlation-request-id'] unless http_response['x-ms-correlation-request-id'].nil?
        result.client_request_id = http_response['x-ms-client-request-id'] unless http_response['x-ms-client-request-id'].nil?
        # Deserialize Response
        if status_code == 200
          begin
            parsed_response = response_content.to_s.empty? ? nil : JSON.load(response_content)
            result_mapper = Azure::Reservations::Mgmt::V2019_07_19_preview::Models::QuotaRequestOneResourceSubmitResponse.mapper()
            result.body = @client.deserialize(result_mapper, parsed_response)
          rescue Exception => e
            fail MsRest::DeserializationError.new('Error occurred in deserializing the response', e.message, e.backtrace, result)
          end
        end
        # Deserialize Response
        if status_code == 201
          begin
            parsed_response = response_content.to_s.empty? ? nil : JSON.load(response_content)
            result_mapper = Azure::Reservations::Mgmt::V2019_07_19_preview::Models::QuotaRequestSubmitResponse201.mapper()
            result.body = @client.deserialize(result_mapper, parsed_response)
          rescue Exception => e
            fail MsRest::DeserializationError.new('Error occurred in deserializing the response', e.message, e.backtrace, result)
          end
        end

        result
      end

      promise.execute
    end

    #
    # Submits a Quota Request for a resource provider at the specified location for
    # the specific resource in the parameter.
    #
    # Submits Quota change request for a resource provider for the specified
    # location for the specific resource in the parameter. To use, first make a Get
    # request to get quota information. This information consists of a list of
    # resources and information regarding those resources. For all the resources in
    # that list which require an update to their quotas, update their limit fields
    # in the response from the Get request to their new values. Then, submit this
    # updated JSON object to this quota request API. This will update the quota to
    # the values specified. The location header in the response will be used to
    # track the status of the quota request. Please check the provisioningState
    # field in the response.
    #
    # @param subscription_id [String] Azure subscription id.
    # @param provider_id [String] Azure resource Provider id.
    # @param location [String] Azure region.
    # @param resource_name [String] The Resource name for the specific resource
    # provider, such as SKU name for Microsoft.Compute, pool for Microsoft.Batch.
    # @param create_quota_request [CurrentQuotaLimitBase] Quota requests payload.
    # @param if_match [String] ETag of the Entity. ETag should match the current
    # entity state from the header response of the GET request or it should be *
    # for unconditional update.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [Object] operation results.
    #
    def begin_update(subscription_id, provider_id, location, resource_name, create_quota_request, if_match, custom_headers:nil)
      response = begin_update_async(subscription_id, provider_id, location, resource_name, create_quota_request, if_match, custom_headers:custom_headers).value!
      response.body unless response.nil?
    end

    #
    # Submits a Quota Request for a resource provider at the specified location for
    # the specific resource in the parameter.
    #
    # Submits Quota change request for a resource provider for the specified
    # location for the specific resource in the parameter. To use, first make a Get
    # request to get quota information. This information consists of a list of
    # resources and information regarding those resources. For all the resources in
    # that list which require an update to their quotas, update their limit fields
    # in the response from the Get request to their new values. Then, submit this
    # updated JSON object to this quota request API. This will update the quota to
    # the values specified. The location header in the response will be used to
    # track the status of the quota request. Please check the provisioningState
    # field in the response.
    #
    # @param subscription_id [String] Azure subscription id.
    # @param provider_id [String] Azure resource Provider id.
    # @param location [String] Azure region.
    # @param resource_name [String] The Resource name for the specific resource
    # provider, such as SKU name for Microsoft.Compute, pool for Microsoft.Batch.
    # @param create_quota_request [CurrentQuotaLimitBase] Quota requests payload.
    # @param if_match [String] ETag of the Entity. ETag should match the current
    # entity state from the header response of the GET request or it should be *
    # for unconditional update.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [MsRestAzure::AzureOperationResponse] HTTP response information.
    #
    def begin_update_with_http_info(subscription_id, provider_id, location, resource_name, create_quota_request, if_match, custom_headers:nil)
      begin_update_async(subscription_id, provider_id, location, resource_name, create_quota_request, if_match, custom_headers:custom_headers).value!
    end

    #
    # Submits a Quota Request for a resource provider at the specified location for
    # the specific resource in the parameter.
    #
    # Submits Quota change request for a resource provider for the specified
    # location for the specific resource in the parameter. To use, first make a Get
    # request to get quota information. This information consists of a list of
    # resources and information regarding those resources. For all the resources in
    # that list which require an update to their quotas, update their limit fields
    # in the response from the Get request to their new values. Then, submit this
    # updated JSON object to this quota request API. This will update the quota to
    # the values specified. The location header in the response will be used to
    # track the status of the quota request. Please check the provisioningState
    # field in the response.
    #
    # @param subscription_id [String] Azure subscription id.
    # @param provider_id [String] Azure resource Provider id.
    # @param location [String] Azure region.
    # @param resource_name [String] The Resource name for the specific resource
    # provider, such as SKU name for Microsoft.Compute, pool for Microsoft.Batch.
    # @param create_quota_request [CurrentQuotaLimitBase] Quota requests payload.
    # @param if_match [String] ETag of the Entity. ETag should match the current
    # entity state from the header response of the GET request or it should be *
    # for unconditional update.
    # @param [Hash{String => String}] A hash of custom headers that will be added
    # to the HTTP request.
    #
    # @return [Concurrent::Promise] Promise object which holds the HTTP response.
    #
    def begin_update_async(subscription_id, provider_id, location, resource_name, create_quota_request, if_match, custom_headers:nil)
      fail ArgumentError, 'subscription_id is nil' if subscription_id.nil?
      fail ArgumentError, 'provider_id is nil' if provider_id.nil?
      fail ArgumentError, 'location is nil' if location.nil?
      fail ArgumentError, 'resource_name is nil' if resource_name.nil?
      fail ArgumentError, '@client.api_version is nil' if @client.api_version.nil?
      fail ArgumentError, 'create_quota_request is nil' if create_quota_request.nil?
      fail ArgumentError, 'if_match is nil' if if_match.nil?


      request_headers = {}
      request_headers['Content-Type'] = 'application/json; charset=utf-8'

      # Set Headers
      request_headers['x-ms-client-request-id'] = SecureRandom.uuid
      request_headers['If-Match'] = if_match unless if_match.nil?
      request_headers['accept-language'] = @client.accept_language unless @client.accept_language.nil?

      # Serialize Request
      request_mapper = Azure::Reservations::Mgmt::V2019_07_19_preview::Models::CurrentQuotaLimitBase.mapper()
      request_content = @client.serialize(request_mapper,  create_quota_request)
      request_content = request_content != nil ? JSON.generate(request_content, quirks_mode: true) : nil

      path_template = 'subscriptions/{subscriptionId}/providers/Microsoft.Capacity/resourceProviders/{providerId}/locations/{location}/serviceLimits/{resourceName}'

      request_url = @base_url || @client.base_url

      options = {
          middlewares: [[MsRest::RetryPolicyMiddleware, times: 3, retry: 0.02], [:cookie_jar]],
          path_params: {'subscriptionId' => subscription_id,'providerId' => provider_id,'location' => location,'resourceName' => resource_name},
          query_params: {'api-version' => @client.api_version},
          body: request_content,
          headers: request_headers.merge(custom_headers || {}),
          base_url: request_url
      }
      promise = @client.make_request_async(:patch, path_template, options)

      promise = promise.then do |result|
        http_response = result.response
        status_code = http_response.status
        response_content = http_response.body
        unless status_code == 200 || status_code == 201
          error_model = JSON.load(response_content)
          fail MsRest::HttpOperationError.new(result.request, http_response, error_model)
        end

        result.request_id = http_response['x-ms-request-id'] unless http_response['x-ms-request-id'].nil?
        result.correlation_request_id = http_response['x-ms-correlation-request-id'] unless http_response['x-ms-correlation-request-id'].nil?
        result.client_request_id = http_response['x-ms-client-request-id'] unless http_response['x-ms-client-request-id'].nil?
        # Deserialize Response
        if status_code == 200
          begin
            parsed_response = response_content.to_s.empty? ? nil : JSON.load(response_content)
            result_mapper = Azure::Reservations::Mgmt::V2019_07_19_preview::Models::QuotaRequestOneResourceSubmitResponse.mapper()
            result.body = @client.deserialize(result_mapper, parsed_response)
          rescue Exception => e
            fail MsRest::DeserializationError.new('Error occurred in deserializing the response', e.message, e.backtrace, result)
          end
        end
        # Deserialize Response
        if status_code == 201
          begin
            parsed_response = response_content.to_s.empty? ? nil : JSON.load(response_content)
            result_mapper = Azure::Reservations::Mgmt::V2019_07_19_preview::Models::QuotaRequestSubmitResponse201.mapper()
            result.body = @client.deserialize(result_mapper, parsed_response)
          rescue Exception => e
            fail MsRest::DeserializationError.new('Error occurred in deserializing the response', e.message, e.backtrace, result)
          end
        end

        result
      end

      promise.execute
    end

  end
end
